@@ -259,6 +259,7 @@ class config(object):
 			self.packages = clone.packages
 			self.repositories = clone.repositories
 			self.unpack_dependencies = clone.unpack_dependencies
+			self._default_features_use = clone._default_features_use
 			self._iuse_effective = clone._iuse_effective
 			self._iuse_implicit_match = clone._iuse_implicit_match
 			self._non_user_variables = clone._non_user_variables
@@ -961,6 +962,14 @@ class config(object):
 
 			# initialize self.features
 			self.regenerate()
+			feature_use = []
+			if "test" in self.features:
+				feature_use.append("test")
+			self.configdict["features"]["USE"] = self._default_features_use = " ".join(feature_use)
+			if feature_use:
+				# Regenerate USE so that the initial "test" flag state is
+				# correct for evaluation of !test? conditionals in RESTRICT.
+				self.regenerate()
 
 			if unprivileged:
 				self.features.add('unprivileged')
@@ -1296,7 +1305,7 @@ class config(object):
 			del self._penv[:]
 			self.configdict["pkg"].clear()
 			self.configdict["pkginternal"].clear()
-			self.configdict["features"].clear()
+			self.configdict["features"]["USE"] = self._default_features_use
 			self.configdict["repo"].clear()
 			self.configdict["defaults"]["USE"] = \
 				" ".join(self.make_defaults_use)
@@ -1598,7 +1607,7 @@ class config(object):
 			has_changed = True
 			# Prevent stale features USE from corrupting the evaluation
 			# of USE conditional RESTRICT.
-			self.configdict["features"]["USE"] = ""
+			self.configdict["features"]["USE"] = self._default_features_use
 
 		self._penv = []
 		cpdict = self._penvdict.get(cp)
@@ -1657,8 +1666,22 @@ class config(object):
 			has_changed = True
 
 		if has_changed:
+			# This can modify self.features due to package.env settings.
 			self.reset(keeping_pkg=1)
 
+		if "test" in self.features:
+			# This is independent of IUSE and RESTRICT, so that the same
+			# value can be shared between packages with different settings,
+			# which is important when evaluating USE conditional RESTRICT.
+			feature_use.append("test")
+
+		feature_use = " ".join(feature_use)
+		if feature_use != self.configdict["features"]["USE"]:
+			# Regenerate USE for evaluation of conditional RESTRICT.
+			self.configdict["features"]["USE"] = feature_use
+			self.reset(keeping_pkg=1)
+			has_changed = True
+
 		if explicit_iuse is None:
 			explicit_iuse = frozenset(x.lstrip("+-") for x in iuse.split())
 		if eapi_attrs.iuse_effective:
@@ -1687,25 +1710,12 @@ class config(object):
 			else:
 				restrict_test = "test" in restrict
 
-		pkginternaluse_before = pkginternaluse
-		if "test" in self.features:
-			# This is independent of IUSE and RESTRICT, so that the same
-			# value can be shared between packages with different settings,
-			# which is important when evaluating USE conditional RESTRICT
-			# above.
-			feature_use.append("test")
-
-			if restrict_test:
-				# Handle it like IUSE="-test", since features USE is
-				# independent of RESTRICT.
-				pkginternaluse_list.append("-test")
-				pkginternaluse = " ".join(pkginternaluse_list)
-				self.configdict["pkginternal"]["USE"] = pkginternaluse
-
-		feature_use = " ".join(feature_use)
-		if (feature_use != self.configdict["features"].get("USE", "") or
-			pkginternaluse is not pkginternaluse_before):
-			self.configdict["features"]["USE"] = feature_use
+		if restrict_test and "test" in self.features:
+			# Handle it like IUSE="-test", since features USE is
+			# independent of RESTRICT.
+			pkginternaluse_list.append("-test")
+			pkginternaluse = " ".join(pkginternaluse_list)
+			self.configdict["pkginternal"]["USE"] = pkginternaluse
 			# TODO: can we avoid that?
 			self.reset(keeping_pkg=1)
 			has_changed = True
